﻿using AsReaderGUN251G;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace AsReaderGUN251GDemo
{
    /// <summary>
    /// BarcodeScan.xaml 的交互逻辑
    /// </summary>
    public partial class BarcodeScan : Page
    {
        private Handler handler = Handler.Instance();

        private readonly string[] rescanTimeArr = { "0", "100", "200", "300", "400", "500", "600","700",
                                   "800", "900", "1000", "1200", "1300", "1400", "1500",
                                   "2000","2500","3000","3500","5000","10000"};

        private static bool isAsReaderTriggerStatus = true;

        private string encodingValue = "";
        private string rescanTimeValue = "";
        private int rescanTimeIndex = 0;
        private bool isJumpPage = true;

        DispatcherTimer dispatcherTimer = new DispatcherTimer();

        public BarcodeScan()
        {
            InitializeComponent();

            Device.receiveAsBridgeIsConnected = receiveAsBridgeIsConnected;
            Device.receiveDeviceConnected = receiveDeviceConnected;
        }

        private void LoadDelegate()
        {
            Device.receiveResultBarcodeScan = ShowBarcodeScanData;
            Device.receiveChangedActionState = StopScanActionState;
            Device.receiveOnAsReaderTriggerKeyEvent = AsReaderTriggerKeyEvent;
            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;

            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);

        }
        private void ClearDelegate()
        {
            Device.receiveResultBarcodeScan = null;
            Device.receiveChangedActionState = null;
            Device.receiveOnAsReaderTriggerKeyEvent = null;
            Device.receiveUpdateDeviceState = null;
        }
        private void BarcodeScan_Loaded(object sender, RoutedEventArgs e) {
            try
            {
                LoadDelegate();
                encodingValue = handler.ReadXmlFile(handler.defineEncodingValue);
                rescanTimeValue = handler.ReadXmlFile(handler.defineRescanTimeValue);

                switch (encodingValue)
                {
                    case "UTF8":
                        this.UTF8.IsChecked = true;
                        break;
                    case "ShiftJS":
                        this.ShiftJS.IsChecked = true;
                        break;
                    default:
                        this.ASCII.IsChecked = true;
                        break;
                }

                this.RescanTimeList.Items.Clear();
                if (rescanTimeArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < rescanTimeArr.Length; i++)
                    {
                        if (i == 0)
                        {
                            this.RescanTimeList.Items.Add("NOT Used");
                        }
                        else
                        {
                            this.RescanTimeList.Items.Add(rescanTimeArr[i] + " ms");
                        }
                    }

                    if (rescanTimeValue != "")
                        rescanTimeIndex = Convert.ToInt32(rescanTimeValue);
                    if (i > 0)
                        this.RescanTimeList.SelectedIndex = rescanTimeIndex;
                }

                barcodeDataArr = Device.barcodeDataArrTmp;
                if (barcodeDataArr.Count > 0)
                {
                    this.BarcodeDataList.ItemsSource = null;
                    this.BarcodeDataList.ItemsSource = barcodeDataArr;
                    this.BarcodeDataList.AutoGenerateColumns = false;  //去掉多余列
                    this.BarcodeDataList.CanUserAddRows = false; //去掉多余行
                    this.BarcodeSumText.Content = barcodeDataArr.Count;
                }              
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: BarcodeScan_Loaded() - " + ex.Message);
            }
        }
        private void Encoding_Checked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.UTF8 != null && this.UTF8.IsChecked == true)
                {
                    encodingValue = "UTF8";
                }
                else if (this.ShiftJS != null && this.ShiftJS.IsChecked == true)
                {
                    encodingValue = "ShiftJS";
                }
                else
                {
                    encodingValue = "ASCII";
                }
                handler.WriteXmlFile(handler.defineEncodingValue, encodingValue);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: Encoding_Checked() - " + ex.Message);
            }
        }

        private void RescanTimeList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                rescanTimeIndex = this.RescanTimeList.SelectedIndex;
                handler.WriteXmlFile(handler.defineRescanTimeValue, rescanTimeIndex.ToString());
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: RescanTimeList_SelectionChanged() - " + ex.Message);
            }
        }

        private void StartScanBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                isCancellLoopScan = true;
                //dispatcherTimer.Start();
                AsPacket.ResultCode resultCode = Device.StartDecode();
                isAsReaderTriggerStatus = true;
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: StartScanBtn_Click() - " + ex.Message);
            }
        }

        private void StopScanBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                StopBarcodeScan();

            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: StopScanBtn_Click() - " + ex.Message);
            }
        }

        private void StopBarcodeScan()
        {
            isAsReaderTriggerStatus = false;
            isCancellLoopScan = true;
            dispatcherTimer.Stop();
            Device.StopDecode();
        }

        private void BarcodeClearBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.BarcodeDataList.ItemsSource = null;
                this.BarcodeSumText.Content = "0";
                barcodeDataArr.Clear();
                Device.barcodeDataArrTmp.Clear();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: BarcodeClearBtn_Click() - " + ex.Message);
            }
        }

        private void BackBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (NavigationService != null)
                {
                    StopBarcodeScan();

                    ClearDelegate();
                    NavigationService.GetNavigationService(this).GoBack();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: BackBtn_Click() - " + ex.Message);
            }
        }

        private void BarcodeTabEventSet(bool status)
        {
            this.StartScanBtn.IsEnabled = status;
            this.BarcodeSetGroup.IsEnabled = status;
            this.BackBtn.IsEnabled = status;
        }

        Dictionary<string, ReceiveBarcodeData> barcodeDataArr = new Dictionary<string, ReceiveBarcodeData>();
        private void ShowBarcodeScanData(string barcodeType, string codeId, byte[] barcodeByte)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    string codeData;

                    if (this.UTF8.IsChecked == true)
                    {
                        codeData = Encoding.UTF8.GetString(barcodeByte);
                    }
                    else if (this.ShiftJS.IsChecked == true)
                    {
                        codeData = Encoding.GetEncoding("Shift-JIS").GetString(barcodeByte);
                    }
                    else
                    {
                        codeData = Encoding.ASCII.GetString(barcodeByte);
                    }

                    string codeKey = barcodeType + codeData;
                    ReceiveBarcodeData barcodeDataChanged;

                    if (barcodeDataArr.ContainsKey(codeKey))
                    {
                        barcodeDataChanged = barcodeDataArr[codeKey];
                        barcodeDataChanged.CountValue = barcodeDataChanged.CountValue + 1;
                        barcodeDataArr[codeKey] = barcodeDataChanged;
                    }
                    else
                    {
                        int nowCount = barcodeDataArr.Count;
                        string TypeCodeIDStr = barcodeType + "\r\n" + codeId;
                        barcodeDataChanged = new ReceiveBarcodeData(nowCount + 1, barcodeType, codeId, codeData, 1, TypeCodeIDStr);
                        barcodeDataArr.Add(codeKey, barcodeDataChanged);
                    }

                    this.BarcodeDataList.ItemsSource = null;//这里是关键
                    this.BarcodeDataList.ItemsSource = barcodeDataArr;
                    this.BarcodeDataList.AutoGenerateColumns = false;  //去掉多余列
                    this.BarcodeDataList.CanUserAddRows = false; //去掉多余行
                    this.BarcodeSumText.Content = barcodeDataArr.Count;
                    Device.barcodeDataArrTmp = barcodeDataArr;

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowBarcodeScanData() - " + ex.Message);
            }
        }

        private static bool isCancellLoopScan = true;
        private void StopScanActionState(AsPacket.CommandType commandType, int resultCode)
        {
            try
            {
                if (commandType == AsPacket.CommandType.CommandDecodeStop)
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        if (isAsReaderTriggerStatus == false || this.RescanTimeList.SelectedIndex == 0)
                        {
                            //BarcodeTabEventSet(true);
                            isCancellLoopScan = true;
                            return;
                        }
                        int selectKey = this.RescanTimeList.SelectedIndex;
                        int timeValue = Convert.ToInt32(rescanTimeArr[selectKey]);
                        if (timeValue != 0)
                        {                           
                            dispatcherTimer.Stop();
                            isCancellLoopScan = false;
                            dispatcherTimer.Interval = new TimeSpan(0, 0, 0, 0, timeValue);
                            dispatcherTimer.IsEnabled = true;
                            dispatcherTimer.Start();

                        }
                    });

                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: StopScanActionState() - " + ex.Message);
            }
        }

        private void dispatcherTimer_Tick(object sender, EventArgs e)
        {
            if (isCancellLoopScan == false)
            {
                if (isAsReaderTriggerStatus == true)
                {
                    isCancellLoopScan = true;
                    Device.StartDecode();
                }
            }
        }

        private bool AsReaderTriggerKeyEvent(Boolean status)
        {
            try
            {
                if (status == true)
                { 
                    isAsReaderTriggerStatus = status;
                    isCancellLoopScan = true;
                    //dispatcherTimer.Start();
                    Device.StartDecode();

                }
                else
                {
                    StopBarcodeScan();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: AsReaderTriggerKeyEvent() - " + ex.Message);
            }

            return false;
        }

        private void ShowUpdateDeviceState(AsPacket.ResultCode resultCode)
        {
            try
            {
                if (resultCode == AsPacket.ResultCode.ResultDeviceIsSleep)
                {
                    Device.isPageReturn = "isSleep";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("");
                }
                if (resultCode == AsPacket.ResultCode.ResultExceptionHandling)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");

                }

                if (resultCode == AsPacket.ResultCode.ResultSendCommandError)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowUpdateDeviceState() - " + ex.Message);
            }
        }

        private void DeviceExceptionStateReturnMainPage(string state)
        {
            isAsReaderTriggerStatus = false;
            isCancellLoopScan = true;
            dispatcherTimer.Stop();
            App.Current.Dispatcher.Invoke((Action)delegate ()
            {
                if (NavigationService != null && isJumpPage == true)
                {
                    isJumpPage = false;
                    if (state == "isDisconnect")
                    {
                        MessageBox.Show("Disconnected.");
                    }
                    NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                }
            });
        }

        private void receiveAsBridgeIsConnected(bool isAsBridgeConnected)
        {
            try
            {
                isAsReaderTriggerStatus = false;
                isCancellLoopScan = true;
                dispatcherTimer.Stop();
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isAsBridgeConnected)
                        {
                            MessageBox.Show("AsBridge Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveAsBridgeIsConnected() - " + ex.Message);
            }
        }

        private void receiveDeviceConnected(bool isDeviceConnected, string accessoryDeviceName)
        {
            try
            {
                isAsReaderTriggerStatus = false;
                isCancellLoopScan = true;
                dispatcherTimer.Stop();
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isDeviceConnected)
                        {
                            MessageBox.Show("251G Device Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveDeviceConnected() - " + ex.Message);
            }
        }

    }
}
