﻿using AsReaderGUN251G;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AsReaderGUN251GDemo
{
    /// <summary>
    /// EPCMask.xaml 的交互逻辑
    /// </summary>
    public partial class EPCMask : Page
    {
        private Handler handler = Handler.Instance();

        private readonly string[] offsetArr = { "0", "16", "32", "64", "80", "96", "112", "128",
                               "144", "160", "176", "192", "208", "224", "240" };
        private readonly string[] lengthArr = { "0", "16", "32", "64", "80", "96", "112", "128",
                               "144", "160", "176", "192", "208", "224", "240" };

        string notMatchingValue = "";
        bool isJumpPage = true;
        private bool isEdit = false;
        EPCMaskItemData SelectedEditRow = null;

        public EPCMask()
        {
            InitializeComponent();

            Device.receiveAsBridgeIsConnected = receiveAsBridgeIsConnected;
            Device.receiveDeviceConnected = receiveDeviceConnected;
        }

        private void LoadDelegate()
        {
            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;
        }
        private void ClearDelegate()
        {
            Device.receiveUpdateDeviceState = null;
        }
        private void EPCMask_Loaded(object sender, RoutedEventArgs e) {
            try
            {
                LoadDelegate();
                notMatchingValue = handler.ReadXmlFile(handler.defineNotMatchingValue);
                switch (notMatchingValue)
                {
                    case "On":
                        this.MatchingOn.IsChecked = true;
                        break;
                    default:
                        this.MatchingOff.IsChecked = true; 
                        break;
                }

                this.OffsetList.Items.Clear();
                if (offsetArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < offsetArr.Length; i++)
                    {
                        this.OffsetList.Items.Add(offsetArr[i] + " bit");
                    }
                    if (i > 0)
                        this.OffsetList.SelectedIndex = 1;
                }

                this.LengthList.Items.Clear();
                if (lengthArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < lengthArr.Length; i++)
                    {
                        this.LengthList.Items.Add(lengthArr[i] + " bit");
                    }
                    if (i > 0)
                        this.LengthList.SelectedIndex = 1;
                }

                handler.ReadFile(handler.defineEPCMaskFilename);
                epcMaskItemArr = Device.epcMaskItemArrTmp;
                if (epcMaskItemArr.Count > 0)
                {
                    SetMaskDataList();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: EPCMask_Loaded() - " + ex.Message);
            }
            
        }
 

        List<EPCMaskItemData> epcMaskItemArr = new List<EPCMaskItemData>();
        private void DoneBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string maskVal = this.MaskText.Text;
                int lengthVal = Convert.ToInt32(lengthArr[this.LengthList.SelectedIndex]);

                if ((maskVal.Length * 4) != lengthVal)
                {
                    MessageBox.Show("Mask length is not valid");
                    return;
                }

                int offsetVal = Convert.ToInt32(offsetArr[this.OffsetList.SelectedIndex]);
                string offsetText = this.OffsetList.Text;
                string lengthText = this.LengthList.Text;

                if (isEdit == true && SelectedEditRow != null
                    && MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    int rowIndex = this.MaskDataList.SelectedIndex;
                    epcMaskItemArr[rowIndex] = new EPCMaskItemData(offsetVal, lengthVal, maskVal, offsetText, lengthText);

                }
                else {
                    EPCMaskItemData maskItem = new EPCMaskItemData(offsetVal, lengthVal, maskVal, offsetText, lengthText);
                    epcMaskItemArr.Add(maskItem);
                }
                

                SetMaskDataList();
                MaskItemSet();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DoneBtn_Click() - " + ex.Message);
            }   

        }

        private void MaskItemSet() {
            isEdit = false;
            SelectedEditRow = null;
            this.EPCMaskTab.IsSelected = true;
            this.MaskText.Text = "";
            this.OffsetList.SelectedIndex = 1;
            this.LengthList.SelectedIndex = 1;
        }
        private void SaveBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Device.epcMaskItemArrTmp = epcMaskItemArr;
                bool isCheckedStatus;
                if (this.MatchingOn.IsChecked == true)
                {
                    notMatchingValue = "On";
                    isCheckedStatus = true;
                }
                else
                {
                    notMatchingValue = "Off";
                    isCheckedStatus = false;
                }
                Device.SetEPCMaskItem(epcMaskItemArr, isCheckedStatus);

                handler.WriteXmlFile(handler.defineNotMatchingValue, notMatchingValue);
                handler.WriteFile(handler.defineEPCMaskFilename);

                if (NavigationService != null)
                {
                    ClearDelegate();
                    if (isJumpPage == true)
                    {
                        Device.isPageReturn = "YES";
                        MessageBox.Show("Saved successfully");
                        NavigationService.GetNavigationService(this).GoBack();
                    }
                    else
                    {
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                    
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: SaveBtn_Click() - " + ex.Message);
            }
            
        }

        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Device.epcMaskItemArrTmp = epcMaskItemArr;
                Device.isPageReturn = "YES";
                ClearDelegate();
                if (NavigationService != null)
                {
                    NavigationService.GetNavigationService(this).GoBack();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: CancelBtn_Click() - " + ex.Message);
            }           
        }

        private void DeleteBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    var SelectedRow = (EPCMaskItemData)MaskDataList.SelectedItem;
                    epcMaskItemArr.Remove(SelectedRow);
                    SetMaskDataList();
                }
                else
                {
                    MessageBox.Show("Please select the row.");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DeleteBtn_Click() - " + ex.Message);
            }            
        }

        private void SetMaskDataList()
        {
            this.MaskDataList.ItemsSource = null;//这里是关键
            this.MaskDataList.ItemsSource = epcMaskItemArr;
            this.MaskDataList.AutoGenerateColumns = false;  //去掉多余列
            this.MaskDataList.CanUserAddRows = false; //去掉多余行
        }

        private void ShowUpdateDeviceState(AsPacket.ResultCode resultCode)
        {
            try
            {
                if (resultCode == AsPacket.ResultCode.ResultDeviceIsSleep)
                {
                    Device.isPageReturn = "isSleep";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("");
                }

                if (resultCode == AsPacket.ResultCode.ResultExceptionHandling)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();                   
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }

                if (resultCode == AsPacket.ResultCode.ResultSendCommandError)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowUpdateDeviceState() - " + ex.Message);
            }
            
        }

        private void MaskText_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            Regex re = new Regex("[^a-fA-F0-9]");
            e.Handled = re.IsMatch(e.Text);
        }

        private void DeviceExceptionStateReturnMainPage(string state) {
            App.Current.Dispatcher.Invoke((Action)delegate ()
            {
                if (NavigationService != null && isJumpPage == true)
                {
                    isJumpPage = false;
                    if (state == "isDisconnect") {
                        MessageBox.Show("Disconnected.");
                    }
                    NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                }
            });
        }

        private void EditBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    this.EPCMaskItemTab.IsSelected = true;
                    isEdit = true;
                    SelectedEditRow = (EPCMaskItemData)MaskDataList.SelectedItem;
                    this.MaskText.Text = SelectedEditRow.Mask;

                    for (int i = 0; i < offsetArr.Length; i++)
                    {
                        if (SelectedEditRow.Offset.ToString() == offsetArr[i])
                        {
                            this.OffsetList.SelectedIndex = i;
                            break;
                        }
                    }
                    for (int i = 0; i < lengthArr.Length; i++)
                    {
                        if (SelectedEditRow.Length.ToString() == lengthArr[i])
                        {
                            this.LengthList.SelectedIndex = i;
                            break;
                        }
                    }
                    
                }
                else
                {
                    MessageBox.Show("Please select the row.");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DeleteBtn_Click() - " + ex.Message);
            }
        }

        private void OnDataGridMouseUp(object o, MouseButtonEventArgs e)
        {
            try
            {
                if (e.OriginalSource is ScrollViewer)
                {
                    ((DataGrid)o).UnselectAll();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: OnDataGridMouseUp() - " + ex.Message);
            }

        }

        private void EpcMaskTab_Click(object o, MouseButtonEventArgs e)
        {
            try
            {
                MaskItemSet();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: SelectionMaskTab_Click() - " + ex.Message);
            }
        }

        private void receiveAsBridgeIsConnected(bool isAsBridgeConnected)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isAsBridgeConnected)
                        {
                            MessageBox.Show("AsBridge Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveAsBridgeIsConnected() - " + ex.Message);
            }
        }

        private void receiveDeviceConnected(bool isDeviceConnected, string accessoryDeviceName)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isDeviceConnected)
                        {
                            MessageBox.Show("251G Device Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveDeviceConnected() - " + ex.Message);
            }
        }

    }

}
