﻿using AsReaderGUN251G;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace AsReaderGUN251GDemo
{
    class Handler
    {
        private readonly string defineDemoXmlName = "AsReaderGUN251GDemo.xml";
        public readonly string defineContinuousModeValue = "CONTINUOUS_MODE_VALUE";     //df_CONTINUOUS_MODE_VALUE
        public readonly string defineMaskTypeValue = "MASK_TYPE_VALUE";                 //#define  df_MASK_TYPE_VALUE
        public readonly string defineEncodingValue = "ENCODING_VALUE";                  //#define df_ENCODING_VALUE
        public readonly string defineRescanTimeValue = "RESCAN_TIME_VALUE";             //#define df_RESCAN_TIME_VALUE
        public readonly string defineSelectFlagValue = "SELECT_FLAG_VALUE";             //#define df_SELECT_FLAG_VALUE
        public readonly string defineNotMatchingValue = "NOT_MATCHING_VALUE";           //#define df_NOT_MATCHING_VALUE    

        public readonly string defineInventorySessionValue = "INVENTORY_SESSION_VALUE"; //#define df_NOT_MATCHING_VALUE    
        public readonly string defineSessionFlagValue = "SESSION_FLAG_VALUE";           //#define df_NOT_MATCHING_VALUE    

        public readonly string defineSelectionMaskFilename = "AsReaderGUN251GSelectionMask.txt";
        public readonly string defineEPCMaskFilename = "AsReaderGUN251GEPCMask.txt";

        private Handler()
        {
        }
        private static Handler instance = null;

        public static Handler Instance()
        {
            if (instance == null)
            {
                instance = new Handler();
            }
            return instance;
        }

        public void CreateXmlFile()
        {
            try
            {
                XDocument xDocument = new XDocument();
                if (!File.Exists(defineDemoXmlName))
                {
                    XElement xRoot = new XElement("Settings");
                    XElement xElement = new XElement(new XElement("Setting", new XElement("key", "FIRST_LOAD"), new XElement("values", "TRUE")));
                    xRoot.Add(xElement);
                    xDocument.Add(xRoot);
                    xDocument.Save(defineDemoXmlName);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: CreateXmlFile() - " + ex.ToString());
            }
        }
        public string ReadXmlFile(string key)
        {
            try
            {
                CreateXmlFile();
                XElement xElement = XElement.Load(defineDemoXmlName);
                IEnumerable<XElement> xElements = from ele in xElement.Elements("Setting") select ele;

                foreach (var item in xElements)
                {
                    if (item.Element("key").Value == key)
                    {
                        return item.Element("values").Value;
                    }

                }
                return "";
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ReadXmlFile(" + key + ") - " + ex.ToString());
                return "";
            }

        }
        public void WriteXmlFile(string key, string value)
        {
            try
            {
                CreateXmlFile();
                XElement xElement = XElement.Load(defineDemoXmlName);
                IEnumerable<XElement> xElements = from ele in xElement.Elements("Setting") select ele;
                bool ExistKey = false;
                foreach (var item in xElements)
                {
                    if (item.Element("key").Value == key)
                    {
                        item.Element("values").Value = value;
                        ExistKey = true;
                        xElement.Save(defineDemoXmlName);
                        return;
                    }
                }
                if (ExistKey == false)
                {
                    AddXmlNode(key, value);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: WriteXmlFile(" + key + "," + value + ") - " + ex.ToString());
            }
        }
        public void AddXmlNode(string key, string value)
        {
            try
            {
                XElement xe = XElement.Load(defineDemoXmlName);
                XElement element = new XElement(new XElement("Setting", new XElement("key", key), new XElement("values", value)));
                xe.Add(element);
                xe.Save(defineDemoXmlName);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: AddXmlNode(" + key + "," + value + ") - " + ex.ToString());
            }
        }

        public void WriteFile(string fileName)
        {
            if (File.Exists(fileName)) {
                File.Delete(fileName);
            }
            StreamWriter sw = new StreamWriter(fileName, false, Encoding.Default);
            if (fileName == defineSelectionMaskFilename) {
                List<SelectionMaskItemData> sMaskItem = Device.selectionMaskItemArrTmp;
                string newline;
                for (int i = 0; i < sMaskItem.Count; i++) {                 
                    newline = (int)sMaskItem[i].Target + "|" + (int)sMaskItem[i].Action + "|" + (int)sMaskItem[i].Bank + "|" 
                              + sMaskItem[i].Offset + "|" + sMaskItem[i].Length + "|" + sMaskItem[i].Mask + "\r\n";
                    sw.Write(newline);
                }

            }

            if (fileName == defineEPCMaskFilename)
            {
                List<EPCMaskItemData> epcMaskItem = Device.epcMaskItemArrTmp;
                string newline;
                for (int i = 0; i < epcMaskItem.Count; i++)
                {
                    newline =  epcMaskItem[i].Offset + "|" + epcMaskItem[i].Length + "|" + epcMaskItem[i].Mask + "\r\n";
                    sw.Write(newline);
                }
            }

            //清空缓冲区
            sw.Flush();
            //关闭流
            sw.Close();

        }

        private readonly string[] targetArr = { "S0", "S1", "S2", "S3", "SL" };
        private readonly string[] bankArr = { "Reserved", "EPC", "TID", "User" };
        public void ReadFile(string fileName)
        {
            if (File.Exists(fileName))
            {
                string[] lines = File.ReadAllLines(fileName, Encoding.Default);
                if (fileName == defineSelectionMaskFilename)
                {
                    List<SelectionMaskItemData> sMaskItem = new List<SelectionMaskItemData>();
                    for (int i = 0; i < lines.Length; i++)
                    {
                        if (lines[i].Length > 0)
                        {
                            string[] nowLine = lines[i].Split('|');
                            if (nowLine.Length < 6) return;
                            AsPacket.MaskTargetType targetVal = (AsPacket.MaskTargetType)Convert.ToInt32(nowLine[0]);
                            AsPacket.MaskActionType actionVal = (AsPacket.MaskActionType)Convert.ToInt32(nowLine[1]);
                            AsPacket.MemoryBank bankVal = (AsPacket.MemoryBank)Convert.ToInt32(nowLine[2]);
                            int offsetVal = Convert.ToInt32(nowLine[3]);
                            int lengthVal = Convert.ToInt32(nowLine[4]);
                            string maskVal = nowLine[5];
                            string targetText = targetArr[(int)targetVal];
                            string actionText = "";
                            string bankText = bankArr[(int)bankVal];
                            switch (actionVal)
                            {
                                case AsPacket.MaskActionType.MaskAction_AB:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Assert SL\r\nNot-Matching is Deassert SL";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Inventoried A\r\nNot-Matching is Inventoried B";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_AN:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Assert SL\r\nNot-Matching is Do Nothing";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Inventoried A\r\nNot-Matching is Do Nothing";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_NB:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Deassert SL";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Inventoried B";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_MN:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Negate SL\r\nNot-Matching is Do Nothing";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Invert Session\r\nNot-Matching is Do Nothing";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_BA:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Deassert SL\r\nNot-Matching is Assert SL";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Inventoried B\r\nNot-Matching is Inventoried A";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_BN:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Deassert SL\r\nNot-Matching is Do Nothing";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Inventoried B\r\nNot-Matching is Do Nothing";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_NA:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Assert SL";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Inventoried A";
                                    }
                                    break;
                                case AsPacket.MaskActionType.MaskAction_NM:
                                    if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Negate SL";
                                    }
                                    else
                                    {
                                        actionText = "Matching is Do Nothing\r\nNot-Matching is Invert Session";
                                    }
                                    break;
                                default:
                                    break;
                            }
                            string targetBankText = targetText + "\r\n" + bankText;
                            string actionMaskText = actionText + "\r\n" + maskVal;
                            string offsetLengthText = offsetVal + " bit" + "\r\n" + lengthVal + " bit";
                            SelectionMaskItemData maskItem = new SelectionMaskItemData(targetVal, actionVal, bankVal, offsetVal, lengthVal, maskVal,
                                                                       targetBankText, actionMaskText, offsetLengthText);
                            sMaskItem.Add(maskItem);
                        }

                    }

                    Device.selectionMaskItemArrTmp = sMaskItem;

                }

                if (fileName == defineEPCMaskFilename)
                {
                    List<EPCMaskItemData> epcMaskItem = new List<EPCMaskItemData>(); ;

                    for (int i = 0; i < lines.Length; i++)
                    {
                        if (lines[i].Length > 0)
                        {
                            string[] nowLine = lines[i].Split('|');
                            if (nowLine.Length < 3) return;
                            int offsetVal = Convert.ToInt32(nowLine[0]);
                            int lengthVal = Convert.ToInt32(nowLine[1]);
                            string maskVal = nowLine[2];

                            EPCMaskItemData maskItem = new EPCMaskItemData(offsetVal, lengthVal, maskVal, offsetVal + " bit", lengthVal + " bit");
                            epcMaskItem.Add(maskItem);
                        }

                    }

                    Device.epcMaskItemArrTmp = epcMaskItem;

                }

            }
            else {
                if (fileName == defineSelectionMaskFilename) {
                    Device.selectionMaskItemArrTmp.Clear();
                }
                if (fileName == defineEPCMaskFilename) {
                    Device.epcMaskItemArrTmp.Clear();
                }
            }

        }

    }
}
