﻿using AsReaderGUN251G;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AsReaderGUN251GDemo
{
    /// <summary>
    /// Inventory.xaml 的交互逻辑
    /// </summary>
    public partial class Inventory : Page
    {
        private Handler handler = Handler.Instance();
        bool isConnected = Device.GetisConnected();
        private readonly string[] maskTypeArr = { "No Mask", "Selection Mask", "EPC Mask" };

        int newReadNumber = 0;
        int isDown = 0;
        DateTime downTime = DateTime.Now;

        string continuousModeValue = "";
        string maskTypeValue = "";
        int maskTypeIndex = 0;

        string selectFlagValue = "";
        string inventorySessionValue = "";
        string sessionFlagValue = "";

        string notMatchingValue = "";
        private bool isJumpPage = true;

        public Inventory()
        {
            InitializeComponent();

            Device.receiveAsBridgeIsConnected = receiveAsBridgeIsConnected;
            Device.receiveDeviceConnected = receiveDeviceConnected;
        }

        private void LoadDelegate()
        {
            Device.receiveResultInventory = ShowInventoryData;
            Device.receiveOnAsReaderTriggerKeyEvent = AsReaderTriggerKeyEvent;
            Device.receiveChangedActionState = StopScanActionState;
            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;
        }
        private void ClearDelegate() {
            Device.receiveResultInventory = null;
            Device.receiveOnAsReaderTriggerKeyEvent = null;
            Device.receiveChangedActionState = null;
            Device.receiveUpdateDeviceState = null;
        }

        private void Inventory_Loaded(object sender, RoutedEventArgs e) {
            try
            {
                LoadDelegate();
                continuousModeValue = handler.ReadXmlFile(handler.defineContinuousModeValue);
                maskTypeValue = handler.ReadXmlFile(handler.defineMaskTypeValue);

                switch (continuousModeValue)
                {
                    case "Off":
                        this.ContinuousOff.IsChecked = true;
                        break;
                    default:
                        this.ContinuousOn.IsChecked = true;
                        break;
                }
                
                this.MaskTypeList.Items.Clear();
                if (maskTypeArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < maskTypeArr.Length; i++)
                    {
                        this.MaskTypeList.Items.Add(maskTypeArr[i]);
                    }

                    if (maskTypeValue != "")
                        maskTypeIndex = Convert.ToInt32(maskTypeValue);

                    if (i > 0)
                        this.MaskTypeList.SelectedIndex = maskTypeIndex;
                }

                inventoryDataArr = Device.inventoryDataArrTmp;
                if (inventoryDataArr.Count > 0)
                {
                    this.CodeDataList.ItemsSource = null;
                    this.CodeDataList.ItemsSource = inventoryDataArr;
                    this.CodeDataList.AutoGenerateColumns = false;  //去掉多余列
                    this.CodeDataList.CanUserAddRows = false; //去掉多余行
                    this.CodeNumText.Content = inventoryDataArr.Count;
                    int codeSumVal = 0;
                    foreach (string tagKey in inventoryDataArr.Keys)
                    {
                        codeSumVal = codeSumVal + inventoryDataArr[tagKey].CountValue;
                    }
                    this.CodeSumText.Content = codeSumVal;
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: Inventory_Loaded() - " + ex.Message);
            }
        }

        private void ContinuousMoude_Checked(object sender, RoutedEventArgs e)
        {
            try
            {
                bool isCheckedStatus;
                if (this.ContinuousOff != null && this.ContinuousOff.IsChecked == true)
                {
                    continuousModeValue = "Off";
                    isCheckedStatus = false;
                }
                else
                {
                    continuousModeValue = "On";
                    isCheckedStatus = true;
                }
                handler.WriteXmlFile(handler.defineContinuousModeValue, continuousModeValue);

                if (isConnected == true && Device.isPageReturn == "NO")
                {
                    Device.SetContinuousMoude(isCheckedStatus);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ContinuousMoude_Checked() - " + ex.Message);
            }
        }

        private void MaskTypeList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            try
            {
                maskTypeIndex = this.MaskTypeList.SelectedIndex;
                handler.WriteXmlFile(handler.defineMaskTypeValue, maskTypeIndex.ToString());

                if (isConnected == true && Device.isPageReturn == "NO")
                {
                    Device.SetMaskType((AsPacket.MaskType)maskTypeIndex);
                    
                    if ((AsPacket.MaskType)maskTypeIndex == AsPacket.MaskType.MaskType_Selection) {
                        selectFlagValue = handler.ReadXmlFile(handler.defineSelectFlagValue);
                        inventorySessionValue = handler.ReadXmlFile(handler.defineInventorySessionValue);
                        sessionFlagValue = handler.ReadXmlFile(handler.defineSessionFlagValue);
                        handler.ReadFile(handler.defineSelectionMaskFilename);

                        List<SelectionMaskItemData> selectionMaskItemArr = Device.selectionMaskItemArrTmp;
                        if (selectFlagValue == "") selectFlagValue = "1";
                        if (inventorySessionValue == "") inventorySessionValue = "0";
                        if (sessionFlagValue == "") sessionFlagValue = "0";

                        Device.SetSelectionMaskItem(selectionMaskItemArr, Convert.ToInt32(selectFlagValue),
                            Convert.ToInt32(inventorySessionValue),Convert.ToInt32(sessionFlagValue));
                    }
                    if ((AsPacket.MaskType)maskTypeIndex == AsPacket.MaskType.MaskType_EPC)
                    {
                        handler.ReadFile(handler.defineEPCMaskFilename);
                        notMatchingValue = handler.ReadXmlFile(handler.defineNotMatchingValue);
                        List<EPCMaskItemData> epcMaskItemArr = Device.epcMaskItemArrTmp;

                        bool isCheckedEPCStatus;
                        if (notMatchingValue == "Off")
                        {
                            isCheckedEPCStatus = false;
                        }
                        else
                        {
                            isCheckedEPCStatus = true;
                        }

                        Device.SetEPCMaskItem(epcMaskItemArr, isCheckedEPCStatus);
                    }
                }
                Device.isPageReturn = "NO";
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: MaskTypeList_SelectionChanged() - " + ex.Message);
            }
        }

        private void InventoryBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                AsPacket.ResultCode resultCode = Device.Inventory();
                if (resultCode == AsPacket.ResultCode.ResultNoError)
                {
                    InventoryTabEventSet(false);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: InventoryBtn_Click() - " + ex.Message);
            }
        }

        private void InventoryTabEventSet(bool status)
        {
            this.InventoryBtn.IsEnabled = status;
            this.InventorySetGroup.IsEnabled = status;
            this.MaskBtn.IsEnabled = status;
            this.BackBtn.IsEnabled = status;
        }

        private void InventoryClearBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                isDown = 0;
                this.CodeDataList.ItemsSource = null;
                this.CodeNumText.Content = "0";
                this.CodeSumText.Content = "0";
                this.TpsText.Content = "0.00 tps";
                inventoryDataArr.Clear();
                Device.inventoryDataArrTmp.Clear();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: InventoryClearBtn_Click() - " + ex.Message);
            }
        }

        private void InventoryStopBtn_Click(object sender, RoutedEventArgs e)
        {
            AsPacket.ResultCode resultCode = Device.Stop();
        }

        private void MaskBtn_Click(object sender, RoutedEventArgs e)
        {
            string MaskText = this.MaskTypeList.Text;
            if (MaskText == "No Mask")
            {
                MessageBox.Show("Please set the mask type in Mask Type menu before using mask.");
                return;
            }
            if (MaskText == "Selection Mask")
            { 
                if (NavigationService != null)
                {
                    ClearDelegate();
                    NavigationService.GetNavigationService(this).Navigate(new Uri("SelectionMask.xaml", UriKind.Relative));
                }     
                return;
            }

            if (MaskText == "EPC Mask")
            {
                if (NavigationService != null)
                {
                    ClearDelegate();
                    NavigationService.GetNavigationService(this).Navigate(new Uri("EPCMask.xaml", UriKind.Relative));
                }
                return;
            }
        }

        Dictionary<string, ReceiveInventoryData> inventoryDataArr = new Dictionary<string, ReceiveInventoryData>();
        private void ShowInventoryData(string tag)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {

                    DateTime nowTime = DateTime.Now;
                    if (isDown == 0)
                    {
                        downTime = nowTime;
                        isDown = 1;
                        newReadNumber = 0;
                    }

                    TimeSpan tsNow = new TimeSpan(nowTime.Ticks);
                    TimeSpan tsDown = new TimeSpan(downTime.Ticks);
                    TimeSpan ts = tsNow.Subtract(tsDown).Duration();
                    int midTime = ts.Hours * 3600 + ts.Minutes * 60 + ts.Seconds;

                    if (midTime == 0)
                    {
                        midTime = 1;
                    }
                    newReadNumber++;
                    string tps = (newReadNumber / midTime).ToString("0.00");
                    this.TpsText.Content = tps + " tps";
                    ReceiveInventoryData InventoryDataChanged;

                    if (inventoryDataArr.ContainsKey(tag))
                    {
                        InventoryDataChanged = inventoryDataArr[tag];
                        InventoryDataChanged.CountValue = InventoryDataChanged.CountValue + 1;
                        inventoryDataArr[tag] = InventoryDataChanged;
                    }
                    else
                    {
                        int nowCount = inventoryDataArr.Count;
                        InventoryDataChanged = new ReceiveInventoryData(nowCount + 1, tag, 1);
                        inventoryDataArr.Add(tag, InventoryDataChanged);
                    }

                    this.CodeDataList.ItemsSource = null;//这里是关键
                    this.CodeDataList.ItemsSource = inventoryDataArr;
                    this.CodeDataList.AutoGenerateColumns = false;  //去掉多余列
                    this.CodeDataList.CanUserAddRows = false; //去掉多余行
                    this.CodeNumText.Content = inventoryDataArr.Count;
                    this.CodeSumText.Content = Convert.ToInt32(this.CodeSumText.Content) + 1;
                    Device.inventoryDataArrTmp = inventoryDataArr;

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowInventoryData() - " + ex.Message);
            }

        }

        private bool AsReaderTriggerKeyEvent(Boolean status)
        {
            try
            {
                if (status == true)
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        if (this.InventoryBtn.IsEnabled == true)
                        {
                            InventoryTabEventSet(false);
                            Device.Inventory();
                        }
                    });

                }
                else
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        Device.Stop();
                    });

                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: AsReaderTriggerKeyEvent() - " + ex.Message);
            }

            return false;
        }

        private void StopScanActionState(AsPacket.CommandType commandType, int resultCode)
        {
            try
            {
                if (commandType == AsPacket.CommandType.CommandStop)
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        isDown = 0;
                        InventoryTabEventSet(true);
                    });
                }

                if (commandType == AsPacket.CommandType.CommandInventory)
                {
                    isDown = 0;
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: StopScanActionState() - " + ex.Message);
            }

        }

        private void ShowUpdateDeviceState(AsPacket.ResultCode resultCode)
        {
            try
            {
                if (resultCode == AsPacket.ResultCode.ResultDeviceIsSleep)
                {
                    Device.isPageReturn = "isSleep";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("");
                }

                if (resultCode == AsPacket.ResultCode.ResultExceptionHandling)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");

                }

                if (resultCode == AsPacket.ResultCode.ResultSendCommandError)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowUpdateDeviceState() - " + ex.Message);
            }
        }

        private void BackBtn_Click(object sender, RoutedEventArgs e)
        {            
            try
            {
                if (NavigationService != null)
                {
                    ClearDelegate();
                    NavigationService.GetNavigationService(this).GoBack();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: BackBtn_Click() - " + ex.Message);
            }
        }

        private void DeviceExceptionStateReturnMainPage(string state)
        {
            App.Current.Dispatcher.Invoke((Action)delegate ()
            {
                if (NavigationService != null && isJumpPage == true)
                {
                    isJumpPage = false;
                    if (state == "isDisconnect")
                    {
                        MessageBox.Show("Disconnected.");
                    }
                    NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                }
            });
        }

        private void receiveAsBridgeIsConnected(bool isAsBridgeConnected)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isAsBridgeConnected)
                        {
                            MessageBox.Show("AsBridge Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveAsBridgeIsConnected() - " + ex.Message);
            }
        }

        private void receiveDeviceConnected(bool isDeviceConnected, string accessoryDeviceName)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isDeviceConnected)
                        {
                            MessageBox.Show("251G Device Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveDeviceConnected() - " + ex.Message);
            }
        }

    }
}
