﻿using AsReaderGUN251G;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using AsBridge_SDK;


namespace AsReaderGUN251GDemo
{
    /// <summary>
    /// Main.xaml 的交互逻辑
    /// </summary>
    public partial class Main : Page
    {
        private Handler handler = Handler.Instance();
        private static AsBridgeManager asBridgeManager = AsBridgeManager.SharedAsBridgeManager();

        bool isConnected = Device.GetisConnected();

        string continuousModeValue = "";
        string maskTypeValue = "";
        int maskTypeIndex = 0;

        string notMatchingValue = "";
        string selectFlagValue = "";
        string inventorySessionValue = "";
        string sessionFlagValue = "";
        private bool isJumpPage = true;
        private bool isLoadSelectBluetooth = true;
        private BluetoothDevice selectBluetooth = Device.selectBluetoothInfo;
        private bool isBluetoothConnected = Device.isBluetoothConnected;
        private int bluetoothConnectedStatus = Device.bluetoothConnectedStatus;
        public Dictionary<string, BluetoothDevice> DeviceList = Device.DeviceList;

        public Main()
        {
            InitializeComponent();

            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;
            asBridgeManager.whenReceivedAsBridgeList = WhenReceivedBluetoothDevice; //代理：接收检索到的蓝牙设备
            Device.receiveAsBridgeIsConnected = receiveAsBridgeIsConnected;
            Device.receiveDeviceConnected = receiveDeviceConnected;
            Device.whenReceivedAsBridgeFirmWareVersion = whenReceivedAsBridgeFirmWareVersion;
            Device.whenReceivedAsBridgeHardWareVersion = whenReceivedAsBridgeHardWareVersion;
            Device.whenReceivedAsBridgeSerialNumber = whenReceivedAsBridgeSerialNumber;
            Device.whenReceivedAsBridgeBattery = whenReceivedAsBridgeBattery;

        }

        private void LoadDelegate()
        {
            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;
        }
        private void ClearDelegate()
        {
            Device.receiveUpdateDeviceState = null;
            asBridgeManager.whenReceivedAsBridgeList = null; //代理：接收检索到的蓝牙设备
            Device.receiveAsBridgeIsConnected = null;
            Device.receiveDeviceConnected = null;
            Device.whenReceivedAsBridgeFirmWareVersion = null;
            Device.whenReceivedAsBridgeHardWareVersion = null;
            Device.whenReceivedAsBridgeSerialNumber = null;
            Device.whenReceivedAsBridgeBattery = null;
        }

        private void Main_Loaded(object sender, RoutedEventArgs e)
        {
            try {
                if (isBluetoothConnected) {
                    Device.GetFirmwareVersion();
                    if (isConnected)
                    {
                        DeviceConnectImg.Source = new BitmapImage(new Uri(@"Images\device_connected.png", UriKind.Relative));
                        DeviceName.Content = Device.connectedDeviceName;
                    }
                    else {
                        DeviceConnectImg.Source = new BitmapImage(new Uri(@"Images\device_disconnected.png", UriKind.Relative));
                        DeviceName.Content = "";
                    }
                    foreach (string bluetooth in DeviceList.Keys)
                    {
                        ListViewItem addItem = new ListViewItem();
                        addItem.Tag = DeviceList[bluetooth].DeviceID;
                        addItem.Content = DeviceList[bluetooth].DeviceName;
                        if (selectBluetooth != null && addItem.Tag != null && selectBluetooth.DeviceID == addItem.Tag.ToString())
                        {
                            isLoadSelectBluetooth = true;
                            addItem.IsSelected = true;
                        }
                        BluetoothList.Items.Add(addItem);
                    }
                }
                AsBridgeIsConnected(isBluetoothConnected);
                this.AppVersionText.Content = "Version " + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
                this.SdkVersionText.Content = "Version " + Device.GetSDKVersion();
                SearchPortNameList();

                ControlIsEnabled();

                if (Device.isPageReturn == "isSleep")
                {
                    DeviceSleepOrWeekUp(false);
                    Device.isPageReturn = "YES";
                    MessageBox.Show("Reader has entered sleep mode. \r\n Press the trigger button to wake up.");                   
                }

                if (Device.isPageReturn == "isDisconnect")
                {
                    ControlIsEnabled();
                    Device.isPageReturn = "YES";
                    this.MassageList.Text += "251G Disconnect. \r\n";
                }
            } catch (Exception ex) {
                this.MassageList.Text += "Error: Loaded - " + ex.Message + "\r\n";
            }
            
        }

        private void SearchPortNameList()
        {
            string[] ports = SerialPort.GetPortNames();
            int i ;
            int selectPortIndex = 0;
            string cachePortName = Device.selectPortName;
            this.ComPortList.Items.Clear();//清除当前存在的端口
            if (ports.Length > 0)
            {
                for (i = 0; i < ports.Length; i++)
                {
                    if (cachePortName == ports[i])
                    {
                        selectPortIndex = i;
                    }
                    this.ComPortList.Items.Add(ports[i]);
                }
                if (i > 0)
                    this.ComPortList.SelectedIndex = selectPortIndex;
            }
        }

        private void SearchBtn_Click(object sender, RoutedEventArgs e)
        {
            try{
                SearchPortNameList();
            }
            catch (Exception ex) {
                this.MassageList.Text += "Error: Search - " + ex.Message + "\r\n";
            }
            
        }
       
        private void ConnectBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (this.ComPortList.SelectedItem == null)
                {
                    this.MassageList.Text += "251G Connection failed. \r\n";
                    return;
                }
                string portName = this.ComPortList.Text;

                Device.selectPortName = portName;
                int ret = Device.ConnectWithPortName(portName);
                if (ret == 1)
                {
                    BluetoothList.IsEnabled = false;
                    isConnected = true;
                    DeviceInitialSetting();
                    ControlIsEnabled();

                    this.MassageList.Text += "251G Connected successfully. \r\n";
                }
                else
                {
                    BluetoothList.IsEnabled = true;
                    isConnected = false;
                    ControlIsEnabled();
                    this.MassageList.Text += "251G Connection failed. \r\n";
                }
            }
            catch (Exception ex)
            {
                this.MassageList.Text += "Error: 251G Connect - " + ex.Message + "\r\n";
            }
            
        }

        private void DeviceInitialSetting() {
            isJumpPage = true;
            continuousModeValue = handler.ReadXmlFile(handler.defineContinuousModeValue);
            maskTypeValue = handler.ReadXmlFile(handler.defineMaskTypeValue);

            notMatchingValue = handler.ReadXmlFile(handler.defineNotMatchingValue);
            selectFlagValue = handler.ReadXmlFile(handler.defineSelectFlagValue);

            inventorySessionValue = handler.ReadXmlFile(handler.defineInventorySessionValue);
            sessionFlagValue = handler.ReadXmlFile(handler.defineSessionFlagValue);

            handler.ReadFile(handler.defineSelectionMaskFilename);
            handler.ReadFile(handler.defineEPCMaskFilename);

            if (maskTypeValue != "")
                maskTypeIndex = Convert.ToInt32(maskTypeValue);
            Device.SetMaskType((AsPacket.MaskType)maskTypeIndex);
            bool isCheckedStatus = true;
            if (continuousModeValue == "Off")
                isCheckedStatus = false;
            Device.SetContinuousMoude(isCheckedStatus);

            List<SelectionMaskItemData> selectionMaskItemArr = Device.selectionMaskItemArrTmp;
            if ((AsPacket.MaskType)maskTypeIndex == AsPacket.MaskType.MaskType_Selection) {
                if (selectFlagValue == "") selectFlagValue = ((int)AsPacket.SelectFlag.SelectFlag_SL).ToString();
                if (inventorySessionValue == "") inventorySessionValue = ((int)AsPacket.SessionType.Session_S0).ToString();
                if (sessionFlagValue == "") sessionFlagValue = ((int)AsPacket.SessionFlag.SessionFlag_AB).ToString();

                Device.SetSelectionMaskItem(selectionMaskItemArr, Convert.ToInt32(selectFlagValue),
                    Convert.ToInt32(inventorySessionValue), Convert.ToInt32(sessionFlagValue));

            }

            List<EPCMaskItemData> epcMaskItemArr = Device.epcMaskItemArrTmp;
            if ((AsPacket.MaskType)maskTypeIndex == AsPacket.MaskType.MaskType_EPC)
            {
                bool isCheckedEPCStatus;
                if (notMatchingValue == "Off")
                {
                    isCheckedEPCStatus = false;
                }
                else
                {
                    isCheckedEPCStatus = true;
                }

                Device.SetEPCMaskItem(epcMaskItemArr, isCheckedEPCStatus);

            }
        }

        private void DisConnectBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                int ret = Device.Disconnect();
                isConnected = false;
                ControlIsEnabled();
                this.MassageList.Text += "251G Disconnect. \r\n";
            }
            catch (Exception ex)
            {
                this.MassageList.Text += "Error: 251G DisConnect - " + ex.Message + "\r\n";
            }
            
        }

        private void ControlIsEnabled()
        {
            //isConnected:true => connect succeed
            this.ConnectBtn.IsEnabled = !isConnected;
            this.DisConnectBtn.IsEnabled = isConnected;
            this.MenuItemList.IsEnabled = isConnected;

        }

        

        private void ShowUpdateDeviceState(AsPacket.ResultCode resultCode) {
            try
            {
                if (resultCode == AsPacket.ResultCode.ResultDeviceIsSleep)
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        DeviceSleepOrWeekUp(false);
                        MessageBox.Show("Reader has entered sleep mode. \r\n Press the trigger button to wake up.");                       
                    });
                }

                if (resultCode == AsPacket.ResultCode.ResultDeviceIsWakeUp)
                {
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        Thread.Sleep(1000);
                        DeviceInitialSetting();
                        DeviceSleepOrWeekUp(true);
                    });
                }
                if (resultCode == AsPacket.ResultCode.ResultExceptionHandling)
                {
                    int ret = Device.Disconnect();
                    isConnected = false;
                    DeviceExceptionStateReturnMainPage("isDisconnect");

                }

                if (resultCode == AsPacket.ResultCode.ResultOpenDeviceError)
                {
                    isConnected = false;
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        ControlIsEnabled();
                    });

                }
                if (resultCode == AsPacket.ResultCode.ResultCloseDeviceError)
                {
                    isConnected = true;
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        ControlIsEnabled();
                    });
                }

                if (resultCode == AsPacket.ResultCode.ResultSendCommandError)
                {
                    int ret = Device.Disconnect();
                    isConnected = false;
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }
                if (resultCode == AsPacket.ResultCode.ResultGetDataTimeOut)
                { 
                    App.Current.Dispatcher.Invoke((Action)delegate ()
                    {
                        MessageBox.Show("Communication Error. Disconnected.");
                        int ret = Device.Disconnect();
                        isConnected = false;

                        ControlIsEnabled();
                        this.MassageList.Text += "Communication Error. Disconnect. \r\n";
                    });
                }
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:ShowUpdateDeviceState - " + ex.Message + "\r\n";
                });
            }
        }

        private void DeviceExceptionStateReturnMainPage(string state)
        {
            App.Current.Dispatcher.Invoke((Action)delegate ()
            {
                if (isJumpPage == true)
                {
                    isJumpPage = false;
                    if (state == "isDisconnect")
                    {
                        MessageBox.Show("Disconnected.");
                    }
                    ControlIsEnabled();
                    this.MassageList.Text += "251G Disconnect. \r\n";
                }
            });

        }

        private void DeviceSleepOrWeekUp(bool status)
        {
            this.MenuItemList.IsEnabled = status;
        }

        private void InventoryMenu_Click(object sender, RoutedEventArgs e)
        {
            ClearDelegate();
            if (NavigationService != null) {
                NavigationService.GetNavigationService(this).Navigate(new Uri("Inventory.xaml", UriKind.Relative));
            }           
        }

        private void BarcodeScanMenu_Click(object sender, RoutedEventArgs e)
        {
            ClearDelegate();
            if (NavigationService != null)
            {
                NavigationService.GetNavigationService(this).Navigate(new Uri("BarcodeScan.xaml", UriKind.Relative));
            }
            
        }

        private void SearchBluetoothBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (SearchBluetoothBtn.Content.ToString() == "Stop Search")
                {
                    SearchBluetoothBtn.Content = "Search";
                    asBridgeManager.StopSearching(); //停止检索蓝牙设备
                }
                else
                {
                    SearchBluetoothBtn.Content = "Stop Search";
                    asBridgeManager.StartSearching(Types.BluetoothType.BluetoothBLE);
                }

            }
            catch (Exception ex)
            {
                this.MassageList.Text += "Error: Search Bluetooth - " + ex.Message + "\r\n";
            }
        }

        private void BluetoothConnectBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (isLoadSelectBluetooth)
                {
                    isLoadSelectBluetooth = false;
                }
                selectBluetooth = new BluetoothDevice();
                if (BluetoothConnectBtn.Content.ToString() == "Connect" && BluetoothList.SelectedItems.Count > 0)
                {
                    ListViewItem addItem = new ListViewItem();
                    ListViewItem row = (ListViewItem)BluetoothList.SelectedItems[0];
                    selectBluetooth = selectBluetooth = DeviceList[row.Tag.ToString()];
                    Device.ConnectWithBluetooth(selectBluetooth);
                }
                Device.selectBluetoothInfo = selectBluetooth;
                if (BluetoothConnectBtn.Content.ToString() == "Disconnect")
                {
                    Device.Disconnect();
                }
                
            }
            catch (Exception ex)
            {
                this.MassageList.Text += "Error: Bluetooth Connect - " + ex.Message + "\r\n";
            }

        }

        /// <summary>
        /// 检索到设备的处理
        /// </summary>
        /// <param name="device">蓝牙设备</param>
        private void WhenReceivedBluetoothDevice(List<BluetoothDevice> devices)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    DeviceList = new Dictionary<string, BluetoothDevice>();
                    for (int i = 0; i < devices.Count; i++)
                    {
                        if (!DeviceList.ContainsKey(devices[i].DeviceID))
                        {
                            DeviceList.Add(devices[i].DeviceID, devices[i]);
                        }
                    }
                    BluetoothList.Items.Clear();
                    foreach (string bluetooth in DeviceList.Keys)
                    {
                        ListViewItem addItem = new ListViewItem();
                        addItem.Tag = DeviceList[bluetooth].DeviceID;
                        addItem.Content = DeviceList[bluetooth].DeviceName;
                        if (isBluetoothConnected && selectBluetooth != null && addItem.Tag != null && selectBluetooth.DeviceID == addItem.Tag.ToString())
                        {
                            isLoadSelectBluetooth = true;
                            addItem.IsSelected = true;
                        }
                        BluetoothList.Items.Add(addItem);
                    }
                    Device.DeviceList = DeviceList;

                });             
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:Bluetooth Device Search - " + ex.Message + "\r\n";
                });
            }

        }

        private void AsBridgeIsConnected(bool isAsBridgeConnected)
        {
            if (isAsBridgeConnected)
            {
                VCPGroup.IsEnabled = false;
                SearchBluetoothBtn.Content = "Search";
                BluetoothConnectBtn.Content = "Disconnect";
                asBridgeManager.StopSearching(); //停止检索蓝牙设备
                this.MassageList.Text += "AsBridge Connected successfully. \r\n";
            }
            else
            {
                VCPGroup.IsEnabled = true;
                BluetoothConnectBtn.Content = "Connect";
                BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery0.png", UriKind.Relative));
                DeviceConnectImg.Source = new BitmapImage(new Uri(@"Images\device_disconnected_default.png", UriKind.Relative));
                DeviceName.Content = "Device";
                FWVersionText.Content = "";
                HDVersionText.Content = "";
                SNVersionText.Content = "";
                this.MenuItemList.IsEnabled = isAsBridgeConnected;
                this.MassageList.Text += "AsBridge Disconnect. \r\n";
            }
        }
        private void receiveAsBridgeIsConnected(bool isAsBridgeConnected) {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    isBluetoothConnected = Device.isBluetoothConnected;
                    bluetoothConnectedStatus = Device.bluetoothConnectedStatus;
                    AsBridgeIsConnected(isAsBridgeConnected);
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:AsBridge Is Connected - " + ex.Message + "\r\n";
                });
            }
        }

        private void receiveDeviceConnected(bool isDeviceConnected, string accessoryDeviceName)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (isDeviceConnected)
                    {
                        DeviceConnectImg.Source = new BitmapImage(new Uri(@"Images\device_connected.png", UriKind.Relative));
                        DeviceName.Content = accessoryDeviceName;
                        this.MassageList.Text += "251G Connected successfully. \r\n";
                    }
                    else
                    {
                        DeviceConnectImg.Source = new BitmapImage(new Uri(@"Images\device_disconnected.png", UriKind.Relative));
                        DeviceName.Content = "Device";
                        this.MassageList.Text += "251G Disconnect. \r\n";
                    }
                    this.MenuItemList.IsEnabled = isDeviceConnected;
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:Device Is Connected - " + ex.Message + "\r\n";
                });
            }
        }
        private void whenReceivedAsBridgeFirmWareVersion(string firmWareVersion)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    FWVersionText.Content = "Version " + firmWareVersion;
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:AsBridge FirmWare Version - " + ex.Message + "\r\n";
                });
            }
        }
        private void whenReceivedAsBridgeHardWareVersion(string hardWareVersion)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    HDVersionText.Content = "Version " + hardWareVersion;
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:AsBridge HardWare Version - " + ex.Message + "\r\n";
                });
            }
        }
        private void whenReceivedAsBridgeSerialNumber(string serialNumber)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    SNVersionText.Content = serialNumber;
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:AsBridge Serial Number - " + ex.Message + "\r\n";
                });
            }
        }
        private void whenReceivedAsBridgeBattery(int battery)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (battery == 0)
                    {
                        BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery0.png", UriKind.Relative));
                    }
                    if (battery == 1)
                    {
                        BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery1.png", UriKind.Relative));
                    }
                    if (battery == 2)
                    {
                        BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery2.png", UriKind.Relative));
                    }
                    if (battery == 3)
                    {
                        BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery3.png", UriKind.Relative));
                    }
                    if (battery == 4)
                    {
                        BatteryImg.Source = new BitmapImage(new Uri(@"Images\battery4.png", UriKind.Relative));
                    }
                });
            }
            catch (Exception ex)
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    this.MassageList.Text += "Error:AsBridge Battery - " + ex.Message + "\r\n";
                });
            }
        }


    }
}
