﻿using AsReaderGUN251G;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace AsReaderGUN251GDemo
{
    /// <summary>
    /// SelectionMask.xaml 的交互逻辑
    /// </summary>
    public partial class SelectionMask : Page
    {
        private Handler handler = Handler.Instance();

        private readonly string[] selectFlagArr = { "SL", "~SL", "ALL" };
        string[] inventorySessionArr = { "S0", "S1", "S2", "S3" };
        string[] sessionFlagArr = { "A Only", "B Only", "A or B" };

        private readonly string[] targetArr = { "S0", "S1", "S2", "S3", "SL" };
        private readonly string[] actionArr = { "AB", "AN", "NB", "MN", "BA", "BN", "NA", "NM" };
        private readonly string[] bankArr = { "Reserved", "EPC", "TID", "User" };
        private readonly string[] offsetArr = { "0", "16", "32", "64", "80", "96", "112", "128",
                               "144", "160", "176", "192", "208", "224", "240" };
        private readonly string[] lengthArr = { "0", "16", "32", "64", "80", "96", "112", "128",
                               "144", "160", "176", "192", "208", "224", "240" };

        private string selectFlagValue = "";
        private int selectFlagIndex = (int)AsPacket.SelectFlag.SelectFlag_SL;

        private string inventorySessionValue = "";
        private int inventorySessionIndex = (int)AsPacket.SessionType.Session_S0;

        private string sessionFlagValue = "";
        private int sessionFlagIndex = (int)AsPacket.SessionFlag.SessionFlag_AB;

        private bool isJumpPage = true;
        private bool isEdit = false;
        SelectionMaskItemData SelectedEditRow = null;

        public SelectionMask()
        {
            InitializeComponent();

            Device.receiveAsBridgeIsConnected = receiveAsBridgeIsConnected;
            Device.receiveDeviceConnected = receiveDeviceConnected;
        }

        private void LoadDelegate()
        {
            Device.receiveUpdateDeviceState = ShowUpdateDeviceState;
        }
        private void ClearDelegate()
        {
            Device.receiveUpdateDeviceState = null;
        }

        private void SelectionMask_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                LoadDelegate();
                selectFlagValue = handler.ReadXmlFile(handler.defineSelectFlagValue);

                this.SelectFlagList.Items.Clear();
                if (selectFlagArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < selectFlagArr.Length; i++)
                    {
                        this.SelectFlagList.Items.Add(selectFlagArr[i]);
                    }
                    if (selectFlagValue != "")
                        selectFlagIndex = Convert.ToInt32(selectFlagValue);
                    if (i > 0)
                        this.SelectFlagList.SelectedIndex = selectFlagIndex - 1;
                }

                this.InventorySessionList.Items.Clear();
                if (inventorySessionArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < inventorySessionArr.Length; i++)
                    {
                        this.InventorySessionList.Items.Add(inventorySessionArr[i]);
                    }
                    if (inventorySessionValue != "")
                        inventorySessionIndex = Convert.ToInt32(inventorySessionValue);
                    if (i > 0)
                        this.InventorySessionList.SelectedIndex = inventorySessionIndex;
                }

                this.SessionFlagList.Items.Clear();
                if (sessionFlagArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < sessionFlagArr.Length; i++)
                    {
                        this.SessionFlagList.Items.Add(sessionFlagArr[i]);
                    }
                    if (sessionFlagValue != "")
                        sessionFlagIndex = Convert.ToInt32(sessionFlagValue);
                    if (i > 0)
                        this.SessionFlagList.SelectedIndex = sessionFlagIndex;
                }

                this.TargetList.Items.Clear();
                if (targetArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < targetArr.Length; i++)
                    {
                        this.TargetList.Items.Add(targetArr[i]);
                    }
                    if (i > 0)
                        this.TargetList.SelectedIndex = 4;
                }

                this.ActionList.Items.Clear();
                if (actionArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < actionArr.Length; i++)
                    {
                        this.ActionList.Items.Add(actionArr[i]);
                    }
                    if (i > 0)
                        this.ActionList.SelectedIndex = 0;
                }

                this.BankList.Items.Clear();
                if (bankArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < bankArr.Length; i++)
                    {
                        this.BankList.Items.Add(bankArr[i]);
                    }
                    if (i > 0)
                        this.BankList.SelectedIndex = 1;
                }

                this.OffsetList.Items.Clear();
                if (offsetArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < offsetArr.Length; i++)
                    {
                        this.OffsetList.Items.Add(offsetArr[i] + " bit");
                    }
                    if (i > 0)
                        this.OffsetList.SelectedIndex = 1;
                }

                this.LengthList.Items.Clear();
                if (lengthArr.Length > 0)
                {
                    int i;
                    for (i = 0; i < lengthArr.Length; i++)
                    {
                        this.LengthList.Items.Add(lengthArr[i] + " bit");
                    }
                    if (i > 0)
                        this.LengthList.SelectedIndex = 1;
                }
                handler.ReadFile(handler.defineSelectionMaskFilename);
                selectionMaskItemArr = Device.selectionMaskItemArrTmp;
                if (selectionMaskItemArr.Count > 0)
                {
                    SetMaskDataList();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: SelectionMask_Loaded() - " + ex.Message);
            }
            
        }

        List<SelectionMaskItemData> selectionMaskItemArr = new List<SelectionMaskItemData>();
        private void DoneBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                string maskVal = this.MaskText.Text;
                int lengthVal = Convert.ToInt32(lengthArr[this.LengthList.SelectedIndex]);

                if ((maskVal.Length * 4) != lengthVal)
                {
                    MessageBox.Show("Mask length is not valid");
                    return;
                }
                AsPacket.MaskTargetType targetVal = (AsPacket.MaskTargetType)this.TargetList.SelectedIndex;
                AsPacket.MaskActionType actionVal = (AsPacket.MaskActionType)this.ActionList.SelectedIndex;
                AsPacket.MemoryBank bankVal = (AsPacket.MemoryBank)this.BankList.SelectedIndex;
                int offsetVal = Convert.ToInt32(offsetArr[this.OffsetList.SelectedIndex]);
                string targetText = this.TargetList.Text;
                string offsetText = this.OffsetList.Text;
                string lengthText = this.LengthList.Text;
                string actionText = this.ActionList.Text;
                string bankText = this.BankList.Text;

                switch (actionVal)
                {
                    case AsPacket.MaskActionType.MaskAction_AB:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Assert SL\r\nNot-Matching is Deassert SL";
                        }
                        else
                        {
                            actionText = "Matching is Inventoried A\r\nNot-Matching is Inventoried B";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_AN:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Assert SL\r\nNot-Matching is Do Nothing";
                        }
                        else
                        {
                            actionText = "Matching is Inventoried A\r\nNot-Matching is Do Nothing";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_NB:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Deassert SL";
                        }
                        else
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Inventoried B";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_MN:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Negate SL\r\nNot-Matching is Do Nothing";
                        }
                        else
                        {
                            actionText = "Matching is Invert Session\r\nNot-Matching is Do Nothing";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_BA:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Deassert SL\r\nNot-Matching is Assert SL";
                        }
                        else
                        {
                            actionText = "Matching is Inventoried B\r\nNot-Matching is Inventoried A";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_BN:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Deassert SL\r\nNot-Matching is Do Nothing";
                        }
                        else
                        {
                            actionText = "Matching is Inventoried B\r\nNot-Matching is Do Nothing";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_NA:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Assert SL";
                        }
                        else
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Inventoried A";
                        }
                        break;
                    case AsPacket.MaskActionType.MaskAction_NM:
                        if (targetVal == AsPacket.MaskTargetType.MaskTarget_SL)
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Negate SL";
                        }
                        else
                        {
                            actionText = "Matching is Do Nothing\r\nNot-Matching is Invert Session";
                        }
                        break;
                    default:
                        break;
                }
                string targetBankText = targetText + "\r\n" + bankText;
                string actionMaskText = actionText + "\r\n" + maskVal;
                string offsetLengthText = offsetText + "\r\n" + lengthText;
                if (isEdit == true && SelectedEditRow != null 
                    && MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    int rowIndex = this.MaskDataList.SelectedIndex;
                    selectionMaskItemArr[rowIndex] = new SelectionMaskItemData(targetVal, actionVal, bankVal, offsetVal, lengthVal, maskVal,
                                                                           targetBankText, actionMaskText, offsetLengthText);
                    
                }
                else {
                    SelectionMaskItemData maskItem = new SelectionMaskItemData(targetVal, actionVal, bankVal, offsetVal, lengthVal, maskVal,
                                                                           targetBankText, actionMaskText, offsetLengthText);
                    selectionMaskItemArr.Add(maskItem);
                }
                

                SetMaskDataList();
                MaskItemSet();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DoneBtn_Click() - " + ex.Message);
            }
        }

        private void MaskItemSet() {
            isEdit = false;
            SelectedEditRow = null;
            this.SelectionMaskTab.IsSelected = true;
            this.MaskText.Text = "";
            this.TargetList.SelectedIndex = 4;
            this.ActionList.SelectedIndex = 0;
            this.BankList.SelectedIndex = 1;
            this.OffsetList.SelectedIndex = 1;
            this.LengthList.SelectedIndex = 1;
        }

        private void SaveBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Device.selectionMaskItemArrTmp = selectionMaskItemArr;
                selectFlagIndex = this.SelectFlagList.SelectedIndex + 1;
                inventorySessionIndex = this.InventorySessionList.SelectedIndex;
                sessionFlagIndex = this.SessionFlagList.SelectedIndex;

                Device.SetSelectionMaskItem(selectionMaskItemArr, selectFlagIndex,
                            inventorySessionIndex, sessionFlagIndex);

                handler.WriteXmlFile(handler.defineSelectFlagValue, selectFlagIndex.ToString());
                handler.WriteXmlFile(handler.defineInventorySessionValue, inventorySessionIndex.ToString());
                handler.WriteXmlFile(handler.defineSessionFlagValue, sessionFlagIndex.ToString());

                handler.WriteFile(handler.defineSelectionMaskFilename);
 
                
                if (NavigationService != null) {
                    ClearDelegate();
                    if (isJumpPage == true)
                    {
                        Device.isPageReturn = "YES";
                        MessageBox.Show("Saved successfully");
                        NavigationService.GetNavigationService(this).GoBack();
                    }
                    else {
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                    
                }
                
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: SaveBtn_Click() - " + ex.Message);
            }
           
        }

        private void CancelBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Device.isPageReturn = "YES";
                ClearDelegate();
                if (NavigationService != null)
                {
                    NavigationService.GetNavigationService(this).GoBack();
                }    
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: CancelBtn_Click() - " + ex.Message);
            }
        }

        private void DeleteBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    var SelectedRow = (SelectionMaskItemData)MaskDataList.SelectedItem;
                    selectionMaskItemArr.Remove(SelectedRow);
                    SetMaskDataList();
                }
                else
                {
                    MessageBox.Show("Please select the row.");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DeleteBtn_Click() - " + ex.Message);
            }
            
        }

        private void SetMaskDataList() {
            this.MaskDataList.ItemsSource = null;//这里是关键
            this.MaskDataList.ItemsSource = selectionMaskItemArr;
            this.MaskDataList.AutoGenerateColumns = false;  //去掉多余列
            this.MaskDataList.CanUserAddRows = false; //去掉多余行

        }

        private void ShowUpdateDeviceState(AsPacket.ResultCode resultCode)
        {
            try
            {
                if (resultCode == AsPacket.ResultCode.ResultDeviceIsSleep)
                {
                    Device.isPageReturn = "isSleep";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("");
                }

                if (resultCode == AsPacket.ResultCode.ResultExceptionHandling)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }

                if (resultCode == AsPacket.ResultCode.ResultSendCommandError)
                {
                    int ret = Device.Disconnect();
                    Device.isPageReturn = "isDisconnect";
                    ClearDelegate();
                    DeviceExceptionStateReturnMainPage("isDisconnect");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: ShowUpdateDeviceState() - " + ex.Message);
            }   
        }

        private void MaskText_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            Regex re = new Regex("[^a-fA-F0-9]");
            e.Handled = re.IsMatch(e.Text);
        }

        private void EditBtn_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                if (MaskDataList.SelectedItems != null && MaskDataList.SelectedItems.Count > 0)
                {
                    this.SelectionMaskItemTab.IsSelected = true;
                    isEdit = true;
                    SelectedEditRow = (SelectionMaskItemData)MaskDataList.SelectedItem;

                    this.MaskText.Text = SelectedEditRow.Mask;
                    this.TargetList.SelectedIndex = Convert.ToInt32((AsPacket.MaskTargetType)SelectedEditRow.Target);
                    this.ActionList.SelectedIndex = Convert.ToInt32((AsPacket.MaskActionType)SelectedEditRow.Action);
                    this.BankList.SelectedIndex = Convert.ToInt32((AsPacket.MemoryBank)SelectedEditRow.Bank);
                    
                    for (int i = 0; i < offsetArr.Length; i++)
                    {
                        if (SelectedEditRow.Offset.ToString() == offsetArr[i])
                        {
                            this.OffsetList.SelectedIndex = i;
                            break;
                        }
                    }
                    for (int i = 0; i < lengthArr.Length; i++)
                    {
                        if (SelectedEditRow.Length.ToString() == lengthArr[i])
                        {
                            this.LengthList.SelectedIndex = i;
                            break;
                        }
                    }
                }
                else {
                    MessageBox.Show("Please select the row.");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: DeleteBtn_Click() - " + ex.Message);
            }
        }

        private void DeviceExceptionStateReturnMainPage(string state)
        {
            App.Current.Dispatcher.Invoke((Action)delegate ()
            {
                if (NavigationService != null && isJumpPage == true)
                {
                    isJumpPage = false;
                    if (state == "isDisconnect")
                    {
                        MessageBox.Show("Disconnected.");
                    }
                    NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                }
            });
        }

        private void OnDataGridMouseUp(object o, MouseButtonEventArgs e)
        {
            try {
                if (e.OriginalSource is ScrollViewer)
                {
                    ((DataGrid)o).UnselectAll();
                }
            } catch (Exception ex) {
                Console.WriteLine("Error: OnDataGridMouseUp() - " + ex.Message);
            }
            
        }

        private void SelectionMaskTab_Click(object o, MouseButtonEventArgs e) {
            try
            {
                MaskItemSet();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: SelectionMaskTab_Click() - " + ex.Message);
            }
        }

        private void receiveAsBridgeIsConnected(bool isAsBridgeConnected)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isAsBridgeConnected)
                        {
                            MessageBox.Show("AsBridge Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }

                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveAsBridgeIsConnected() - " + ex.Message);
            }
        }

        private void receiveDeviceConnected(bool isDeviceConnected, string accessoryDeviceName)
        {
            try
            {
                App.Current.Dispatcher.Invoke((Action)delegate ()
                {
                    if (NavigationService != null && isJumpPage == true)
                    {
                        isJumpPage = false;
                        if (!isDeviceConnected)
                        {
                            MessageBox.Show("251G Device Disconnected.");
                        }
                        NavigationService.GetNavigationService(this).Navigate(new Uri("Main.xaml", UriKind.Relative));
                    }
                });
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: receiveDeviceConnected() - " + ex.Message);
            }
        }

    }
}
